@@ -3,11 +3,12 @@ use strict;
 use warnings;
 use 5.010; # state
 # ABSTRACT: turn on Unicode - all of it
-our $VERSION = '0.008'; # VERSION
+our $VERSION = '0.012'; # VERSION
 
 
 use Import::Into;
 use parent qw(Encode charnames utf8 open warnings feature);
+use Symbol qw/qualify_to_ref/;
 
 sub import {
     my $target = caller;
@@ -20,7 +21,7 @@ sub import {
 
     {
         no strict qw(refs); ## no critic (TestingAndDebugging::ProhibitNoStrict)
-        *{$target . '::readdir'} = \&_utf8_readdir;
+        *{$target . '::readdir'} = \&_utf8_readdir unless $^O eq 'Win32';
     }
 
     # utf8 in @ARGV
@@ -38,15 +39,18 @@ sub _encode_argv {
 }
 
 sub _utf8_readdir(*) { ## no critic (Subroutines::ProhibitSubroutinePrototypes)
-    my $handle = shift;
+    my $pre_handle = shift;
+    my $handle = ref($pre_handle) ? $pre_handle : qualify_to_ref($pre_handle, caller);
+    my $hints = (caller 0)[10];
     if (wantarray) {
         my @all_files  = CORE::readdir($handle);
+        return @all_files if not $hints->{'utf8::all'};
         $_ = Encode::decode('UTF-8', $_) for @all_files;
         return @all_files;
     }
     else {
         my $next_file = CORE::readdir($handle);
-        $next_file = Encode::decode('UTF-8', $next_file);
+        $next_file = Encode::decode('UTF-8', $next_file) if $hints->{'utf8::all'};
         return $next_file;
     }
 }
@@ -55,9 +59,10 @@ sub _utf8_readdir(*) { ## no critic (Subroutines::ProhibitSubroutinePrototypes)
 1;
 
 __END__
+
 =pod
 
-=encoding utf-8
+=encoding UTF-8
 
 =head1 NAME
 
@@ -65,7 +70,7 @@ utf8::all - turn on Unicode - all of it
 
 =head1 VERSION
 
-version 0.008
+version 0.012
 
 =head1 SYNOPSIS
 
@@ -147,4 +152,3 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
